@@ -1,6 +1,6 @@
 import cgi
 import os
-import subprocess
+import os.path
 from PIL import Image
 from lxml import etree
 from lxml.etree import XMLSyntaxError
@@ -8,29 +8,20 @@ from lxml.etree import XMLSyntaxError
 from collections import namedtuple, defaultdict
 from zipfile import ZipFile, BadZipfile
 
+from docx2html.exceptions import (
+    ConversionFailed,
+    FileNotDocx,
+    MalformedDocx,
+)
+
 DETECT_FONT_SIZE = False
 EMUS_PER_PIXEL = 9525
-# Abiword supported formats
-VALID_EXTRACT_EXTENSIONS = [
-    '.doc', '.docx', '.dotx', '.docm', '.dotm', '.wri', '.rtf', '.txt',
-    '.text', '.wpd', '.wp', '.odt', '.ott', '.abw', '.atw', '.pdf', '.html',
-    '.dot',
-]
 
 ###
 # Help functions
 ###
 
 
-def is_extractable(path):
-    """
-    Determine if a file is something that we can extract.
-    """
-    _, extension = os.path.splitext(path)
-    extension = extension.lower()
-    return (extension in VALID_EXTRACT_EXTENSIONS)
-
-
 def replace_ext(file_path, new_ext):
     """
     >>> replace_ext('one/two/three.four.doc', '.html')
@@ -1153,6 +1144,9 @@ def get_p_data(p, meta_data, is_td=False):
                 p_text += '<br />'
             else:  # We have an image
                 image_id = get_image_id(child)
+                if image_id not in meta_data.relationship_dict:
+                    # This image does not have an image_id
+                    continue
                 src = meta_data.image_handler(
                     image_id,
                     meta_data.relationship_dict,
@@ -1177,19 +1171,30 @@ def get_zip_file_handler(file_path):
     return ZipFile(file_path)
 
 
-def convert(file_path, image_handler=None, fall_back=None):
+def read_html_file(file_path):
+    with open(file_path) as f:
+        html = f.read()
+    return html
+
+
+def convert(file_path, image_handler=None, fall_back=None, converter=None):
+    """
+    ``file_path`` is a path to the file on the file system that you want to be
+        converted to html.
+    ``image_handler`` is a function that takes an image_id and a
+        relationship_dict to generate the src attribute for images. (see readme
+        for more details)
+    ``fall_back`` is a function that takes a ``file_path``. This function will
+        only be called if for whatever reason the conversion fails.
+    ``converter`` is a function to convert a document that is not docx to docx
+        (examples in docx2html.converters)
+
+    Returns html extracted from ``file_path``
+    """
     file_base, extension = os.path.splitext(os.path.basename(file_path))
 
-    if not is_extractable(file_path):
-        #XXX create better exception, used to be InvalidFileExtension
-        raise Exception(
-            'The file type "%s" is not supported' % extension
-        )
-
-    if extension == '.html':
-        with open(file_path) as f:
-            html = f.read()
-        return html
+    if extension == '.html' or extension == '.htm':
+        return read_html_file(file_path)
 
     # Create the converted file as a file in the same dir with the
     # same name only with a .docx extension
@@ -1198,25 +1203,20 @@ def convert(file_path, image_handler=None, fall_back=None):
         # If the file is already html, just leave it in place.
         docx_path = file_path
     else:
-        # Convert the file to docx
-        # TODO make this configurable.
-        subprocess.call(
-            ['abiword', '--to=docx', '--to-name', docx_path, file_path],
-        )
+        if converter is None:
+            raise FileNotDocx('The file passed in is not a docx.')
+        converter(docx_path, file_path)
+        if not os.path.isfile(docx_path):
+            if fall_back is None:
+                raise ConversionFailed('Conversion to docx failed.')
+            else:
+                return fall_back(file_path)
+
     try:
         # Docx files are actually just zip files.
         zf = get_zip_file_handler(docx_path)
     except BadZipfile:
-        # If its a malformed zip file raise InvalidFileExtension
-        # XXX
-        raise Exception('This file is not a docx')
-    except IOError:
-        # This means that the conversion from abiword failed.
-        if fall_back is not None:
-            return fall_back(file_path)
-        else:
-            # XXX
-            raise Exception('Conversion to docx failed.')
+        raise MalformedDocx('This file is not a docx')
 
     # Need to populate the xml based on word/document.xml
     tree, meta_data = _get_document_data(zf, image_handler)
