@@ -1,8 +1,10 @@
+import mock
 import tempfile
 import shutil
 from os import path
 from zipfile import ZipFile
 from nose.plugins.skip import SkipTest
+from nose.tools import assert_raises
 
 from docx2html.tests import collapse_html
 from docx2html import convert
@@ -10,6 +12,9 @@ from docx2html.core import (
     _get_document_data,
     DETECT_FONT_SIZE,
 )
+from docx2html.exceptions import (
+    ConversionFailed,
+)
 
 
 def assert_html_equal(actual_html, expected_html):
@@ -693,3 +698,50 @@ def test_has_title():
     )
     actual_html = convert(file_path)
     assert_html_equal(actual_html, '''<html><p>Text</p></html>''')
+
+
+def _converter(*args, **kwargs):
+    # Having a converter that does nothing is the same as if abiword fails to
+    # convert.
+    pass
+
+
+def test_converter_broken():
+    file_path = 'test.doc'
+    assert_raises(
+        ConversionFailed,
+        lambda: convert(file_path, converter=_converter),
+    )
+
+
+def test_fall_back():
+    file_path = 'test.doc'
+
+    def fall_back(*args, **kwargs):
+        return 'success'
+    html = convert(file_path, fall_back=fall_back, converter=_converter)
+    assert html == 'success'
+
+
+@mock.patch('docx2html.core.read_html_file')
+@mock.patch('docx2html.core.get_zip_file_handler')
+def test_html_files(patch_zip_handler, patch_read):
+    def raise_assertion(*args, **kwargs):
+        raise AssertionError('Should not have called get_zip_file_handler')
+    patch_zip_handler.side_effect = raise_assertion
+
+    def return_text(*args, **kwargs):
+        return 'test'
+    patch_read.side_effect = return_text
+
+    # Try with an html file
+    file_path = 'test.html'
+
+    html = convert(file_path)
+    assert html == 'test'
+
+    # Try again with an htm file.
+    file_path = 'test.htm'
+
+    html = convert(file_path)
+    assert html == 'test'
